/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qstring.h>
#include <qregexp.h>
#include <qptrlist.h>

#include <bibtexencoder.h>

BibTeXEncoder *bibTeXEncoder = NULL;

static const struct BibTeXEncoderCharMapping
{
    const char *regexp;
    unsigned int unicode;
    const char *latex;
}
charmappingdata[] = {
                        {"\\\\#", 0x0023, "\\#"},
                        {"\\\\&", 0x0026, "\\&"},
//                         {"\\{~\\}", 0x007E, "{~}"},
                        {"!`", 0x00A1, "!`"},
                        {"\"<", 0x00AB, "\"<"},
                        {"\">", 0x00BB, "\">"},
                        {"?`", 0x00BF, "?`"},
                        {"\\{?\\\\`\\{?A\\}?\\}?", 0x00C0, "\\`A"},
                        {"\\{?\\\\'\\{?A\\}?\\}?", 0x00C1, "\\'A"},
                        {"\\{?\\\\\\^\\{?A\\}?\\}?", 0x00C21, "\\^A"},
                        {"\\{?\\\\~\\{?A\\}?\\}?", 0x00C3, "\\~A"},
                        {"\\{?\\\\\"\\{?A\\}?\\}?", 0x00C4, "{\\\"A}"},
                        {"\\{?\\\\AA(\\{\\})?\\}?", 0x00C5, "\\r{A}"},
                        {"\\{?\\\\r\\{?A\\}?\\}?", 0x00C5, "\\r{A}"},
                        {"\\{?\\\\AE\\}?(\\{\\})?", 0x00C6, "{\\AE}"},
                        {"\\{?\\\\c\\{?C\\}?\\}?", 0x00C7, "\\c{C}"},
                        {"\\{?\\\\`\\{?E\\}?\\}?", 0x00C8, "\\`E"},
                        {"\\{?\\\\'\\{?E\\}?\\}?", 0x00C9, "\\'E"},
                        {"\\{?\\\\\\^\\{?E\\}?\\}?", 0x00CA, "\\^E"},
                        {"\\{?\\\\\"\\{?E\\}?\\}?", 0x00CB, "{\\\"E}"},
                        {"\\{?\\\\'\\{?I\\}?\\}?", 0x00CD, "\\'I"},
                        {"\\{?\\\\\\^\\{?I\\}?\\}?", 0x00CE, "\\^I"},
                        {"\\{?\\\\~\\{?N\\}?\\}?", 0x00D1, "\\~N"},
                        {"\\{?\\\\`\\{?O\\}?\\}?", 0x00D2, "\\`O"},
                        {"\\{?\\\\'\\{?O\\}?\\}?", 0x00D3, "\\'O"},
                        {"\\{?\\\\\\^\\{?O\\}?\\}?", 0x00D4, "\\^O"},
                        {"\\{?\\\\\"\\{?O\\}?\\}?", 0x00D6, "{\\\"O}"},
                        {"\\{?\\\\{?O\\}?\\}?", 0x00D8, "{\\O}"},
                        {"\\{?\\\\`\\{?U\\}?\\}?", 0x00D9, "\\`U"},
                        {"\\{?\\\\'\\{?U\\}?\\}?", 0x00DA, "\\'U"},
                        {"\\{?\\\\\\^\\{?U\\}?\\}?", 0x00DB, "\\^U"},
                        {"\\{?\\\\\"\\{?U\\}?\\}?", 0x00DC, "{\\\"U}"},
                        {"\\{?\\\\'\\{?Y\\}?\\}?", 0x00DD, "\\'Y"},
                        {"\\{?\\\\ss\\}?(\\{\\})?", 0x00DF, "{\\ss}"},
                        {"\\{?\\\\`\\{?a\\}?\\}?", 0x00E0, "\\`a"},
                        {"\\{?\\\\'\\{?a\\}?\\}?", 0x00E1, "\\'a"},
                        {"\\{?\\\\\\^\\{?a\\}?\\}?", 0x00E2, "\\^a"},
                        {"\\{?\\\\~\\{?a\\}?\\}?", 0x00E3, "\\~a"},
                        {"\\{?\\\\\"\\{?a\\}?\\}?", 0x00E4, "{\\\"a}"},
                        {"\\{?\\\\aa(\\{\\})?\\}?", 0x00E5, "\\r{a}"},
                        {"\\{?\\\\r\\{?a\\}?\\}?", 0x00E5, "\\r{a}"},
                        {"\\{?\\\\ae\\}?(\\{\\})?", 0x00E6, "{\\ae}"},
                        {"\\{?\\\\c\\{?c\\}?\\}?", 0x00E7, "\\c{c}"},
                        {"\\{?\\\\`\\{?e\\}?\\}?", 0x00E8, "\\`e"},
                        {"\\{?\\\\'\\{?e\\}?\\}?", 0x00E9, "\\'e"},
                        {"\\{?\\\\\\^\\{?e\\}?\\}?", 0x00EA, "\\^e"},
                        {"\\{?\\\\\"\\{?e\\}?\\}?", 0x00EB, "{\\\"e}"},
                        {"\\{?\\\\'\\{?i\\}?\\}?", 0x00ED, "\\'i"},
                        {"\\{?\\\\\\^\\{?i\\}?\\}?", 0x00EE, "\\^i"},
                        {"\\{?\\\\~\\{?n\\}?\\}?", 0x00F1, "\\~n"},
                        {"\\{?\\\\`\\{?o\\}?\\}?", 0x00F2, "\\`o"},
                        {"\\{?\\\\'\\{?o\\}?\\}?", 0x00F3, "\\'o"},
                        {"\\{?\\\\\\^\\{?o\\}?\\}?", 0x00F4, "\\^o"},
                        {"\\{?\\\\\"\\{?o\\}?\\}?", 0x00F6, "{\\\"o}"},
                        {"\\{?\\\\\\{?o\\}?\\}?", 0x00F8, "{\\o}"},
                        {"\\{?\\\\`\\{?u\\}?\\}?", 0x00F9, "\\`u"},
                        {"\\{?\\\\'\\{?u\\}?\\}?", 0x00FA, "\\'u"},
                        {"\\{?\\\\\\^\\{?u\\}?\\}?", 0x00FB, "\\^u"},
                        {"\\{?\\\\\"\\{?u\\}?\\}?", 0x00FC, "{\\\"u}"},
                        {"\\{?\\\\'\\{?y\\}?\\}?", 0x00FD, "\\'y"},
                        {"\\{?\\\\u\\{?A\\}?\\}?", 0x0102, "\\u{A}"},
                        {"\\{?\\\\u\\{?a\\}?\\}?", 0x0103, "\\u{a}"},
                        {"\\{?\\\\'\\{?C\\}?\\}?", 0x0106, "\\'C"},
                        {"\\{?\\\\'\\{?c\\}?\\}?", 0x0107, "\\'c"},
                        {"\\{?\\\\v\\{?C\\}?\\}?", 0x010C, "\\v{C}"},
                        {"\\{?\\\\v\\{?c\\}?\\}?", 0x010D, "\\v{c}"},
                        {"\\{?\\\\L\\}?(\\{\\})?", 0x0141, "{\\L}"},
                        {"\\{?\\\\l\\}?(\\{\\})?", 0x0142, "{\\l}"},
                        {"\\{?\\\\'\\{?N\\}?\\}?", 0x0143, "\\'N"},
                        {"\\{?\\\\'\\{?n\\}?\\}?", 0x0144, "\\'n"},
                        {"\\{?\\\\H\\{?O\\}?\\}?", 0x0150, "\\H{O}"},
                        {"\\{?\\\\H\\{?o\\}?\\}?", 0x0151, "\\H{o}"},
                        {"\\{?\\\\OE\\}?(\\{\\})?", 0x0152, "{\\OE}"},
                        {"\\{?\\\\oe\\}?(\\{\\})?", 0x0153, "{\\oe}"},
                        {"\\{?\\\\v\\{?S\\}?\\}?", 0x0160, "\\v{S}"},
                        {"\\{?\\\\v\\{?s\\}?\\}?", 0x0161, "\\v{s}"},
                        {"\\{?\\\\r\\{?U\\}?\\}?", 0x016E, "\\r{U}"},
                        {"\\{?\\\\r\\{?u\\}?\\}?", 0x016F, "\\r{u}"},
                        {"\\{?\\\\v\\{?A\\}?\\}?", 0x01CD, "\\v{A}"},
                        {"\\{?\\\\v\\{?a\\}?\\}?", 0x01CE, "\\v{a}"},
                        {"--", 0x2013, "--"}
                    };
static const int charmappingdatacount = sizeof( charmappingdata ) / sizeof( charmappingdata[ 0 ] ) ;

BibTeXEncoder::BibTeXEncoder()
{
    buildCharMapping();
}

BibTeXEncoder::~BibTeXEncoder()
{
    freeCharMapping();
}

QString BibTeXEncoder::decode( const QString & text )
{
    QString result = text;

    // remove trailing brackets
    while ( ( result.at( 0 ) == '{' ) && ( result.at( result.length() - 1 ) == '}' ) )
        result = result.mid( 1, result.length() - 2 );

    for ( CharMappingItem * charMappingItem = charMapping->first(); charMappingItem; charMappingItem = charMapping->next() )
        result.replace( *( charMappingItem->regExp ), *( charMappingItem->unicode ) );

    //qDebug( "decoding \"%s\" to \"%s\"", text.latin1(), result.latin1() );
    return result;
}

QString BibTeXEncoder::encode( const QString & text )
{
    QString result = text;

    for ( CharMappingItem * charMappingItem = charMapping->first(); charMappingItem; charMappingItem = charMapping->next() )
        result.replace( *( charMappingItem->unicode ), *( charMappingItem->latex ) );

    //qDebug( "encoding \"%s\" to \"%s\"", text.latin1(), result.latin1() );
    return result;
}

QString BibTeXEncoder::decodeSpecialized( const QString & text, const BibTeXEntryField::FieldType fieldType )
{
    QString result = decode( text );

    switch ( fieldType )
    {
    case BibTeXEntryField::ftPages:
        result.replace( "--", QChar( 0x2013 ) );
        result.replace( "-", QChar( 0x2013 ) );
        break;
    default:
        break;
    }

    return result;
}

QString BibTeXEncoder::encodeSpecialized( const QString & text, const BibTeXEntryField::FieldType fieldType )
{
    QString result = encode( text );

    switch ( fieldType )
    {
    case BibTeXEntryField::ftPages:
        result.replace( QChar( 0x2013 ), "--" );
        break;
    default:
        break;
    }

    return result;
}

void BibTeXEncoder::buildCharMapping()
{
    charMapping = new QPtrList<CharMappingItem>();

    for ( int i = 0; i < charmappingdatacount; i++ )
    {
        CharMappingItem * charMappingItem = new CharMappingItem;
        charMappingItem->regExp = new QRegExp( charmappingdata[ i ].regexp );
        charMappingItem->unicode = new QChar( charmappingdata[ i ].unicode );
        charMappingItem->latex = new QString( charmappingdata[ i ].latex );
        charMapping->append( charMappingItem );
    }
}

void BibTeXEncoder::freeCharMapping()
{
    CharMappingItem * charMappingItem = NULL;

    while ( ( charMappingItem = charMapping->first() ) != NULL )
    {
        delete charMappingItem->regExp;
        delete charMappingItem->unicode;
        delete charMappingItem->latex;
        charMapping->remove();
        delete charMappingItem;
    }

    delete charMapping;
    charMapping = NULL;
}

BibTeXEncoder *BibTeXEncoder::currentBibTeXEncoder()
{
    if ( bibTeXEncoder == NULL )
        bibTeXEncoder = new BibTeXEncoder();

    return bibTeXEncoder;
}
