/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <bibtexencoder.h>
#include "bibtexpreamble.h"

BibTeXPreamble::BibTeXPreamble( const QString &preamble, QObject *parent, const char *name )
        : BibTeXElement( parent, name ), m_preamble( BibTeXEncoder::currentBibTeXEncoder() ->decode( preamble ) )
{}

BibTeXPreamble::BibTeXPreamble( BibTeXPreamble *other )
{
    m_preamble = other->m_preamble;
}

BibTeXPreamble::~BibTeXPreamble()
{}

void BibTeXPreamble::setPreamble( const QString &preamble )
{
    m_preamble = BibTeXEncoder::currentBibTeXEncoder() ->decode( preamble );
}

QString BibTeXPreamble::preamble()
{
    return m_preamble;
}

QString BibTeXPreamble::toString( BibTeXFile::Format format, BibTeXFile::Encoding encoding )
{
    switch ( format )
    {
    case BibTeXFile::formatBibTeX:
        switch ( encoding )
        {
        case BibTeXFile::encImplicit:
        case BibTeXFile::encLaTeX :
            return QString( "@preamble {" ).append( BibTeXEncoder::currentBibTeXEncoder() ->encode( m_preamble ) ).append( "}" );
        case BibTeXFile::encUTF8 :
            return QString( "@preamble {" ).append( m_preamble ).append( "}" );
        default:
            break;
        }
        break;
    case BibTeXFile::formatXML:
        return QString( " <preamble>" ).append( m_preamble ).append( "</preamble>" );
        break;
    default:
        break;
    }

    return QString::null;
}

bool BibTeXPreamble::containsPattern( const QString& pattern, bool caseSensitive )
{
    return m_preamble.find( pattern, 0, caseSensitive ) > 0;
}

#include "bibtexpreamble.moc"
