// $id$

/***************************************************************************
 *   Copyright (C) 2004-2005 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <bibtexencoder.h>
#include "bibtexstring.h"

#define max(a,b) ((a)>(b)?(a):(b))

BibTeXString::BibTeXString( const QString &key, const QString &value, QObject *parent, const char *name )
        : BibTeXElement( parent, name ), m_key( key ), m_value( BibTeXEncoder::currentBibTeXEncoder() ->decode( value ) )
{
    //TODO
}

BibTeXString::BibTeXString( BibTeXString *other )
{
    m_key = other->m_key;
    m_value = other->m_value;
}

BibTeXString::~BibTeXString()
{
    //TODO
}

void BibTeXString::setKey( const QString &key )
{
    m_key = key;
}
QString BibTeXString::key()

{
    return m_key;
}
void BibTeXString::setValue( const QString &value )
{
    m_value = BibTeXEncoder::currentBibTeXEncoder() ->decode( value );
}
QString BibTeXString::value()
{
    return m_value;
}

QString BibTeXString::toString( BibTeXFile::Format format, BibTeXFile::Encoding encoding )
{
    switch ( format )
    {
    case BibTeXFile::formatBibTeX:
        {
            QString text;
            if ( encoding == BibTeXFile::encUTF8 )
                text = m_value;
            else
                text = BibTeXEncoder::currentBibTeXEncoder() ->encode( m_value );

            QString result = QString( "@string { " ).append( m_key ).append( " = " );

            BibTeXFile::StringProtection sp = BibTeXFile::spQuote;
            if ( text.contains( '"' ) > 0 )
                sp = BibTeXFile::spParanthesis;

            if ( sp == BibTeXFile::spParanthesis )
            {
                int count = 0;
                for ( unsigned int i = 0; i < text.length(); i++ )
                {
                    if ( text[ i ] == '}' && ( i == 0 || text[ i - 1 ] != '\\' ) )
                        count++;
                    if ( text[ i ] == '{' && ( i == 0 || text[ i - 1 ] != '\\' ) )
                        count--;
                }
                count = max( 0, count ) + 1;
                for ( int i = 0;i < count; i++ )
                    result.append( "{" );
            }
            else result.append( "\"" );

            result.append( text );

            if ( sp == BibTeXFile::spParanthesis )
            {
                int count = 0;
                for ( unsigned int i = 0; i < text.length(); i++ )
                {
                    if ( text[ i ] == '{' && ( i == 0 || text[ i - 1 ] != '\\' ) )
                        count++;
                    if ( text[ i ] == '}' && ( i == 0 || text[ i - 1 ] != '\\' ) )
                        count--;
                }
                count = max( 0, count ) + 1;
                for ( int i = 0;i < count; i++ )
                    result.append( "}" );
            }
            else result.append( "\"" );

            result.append( " }" );
        }
        break;
    case BibTeXFile::formatXML:
        return QString( " <macro alias=\"%1\">" ).arg( m_key ).append( m_value ).append( "</macro>" );
        break;
    default:
        break;
    }

    return QString::null;
}

bool BibTeXString::containsPattern( const QString& pattern, bool caseSensitive )
{
    return ( m_key.find( pattern, 0, caseSensitive ) > 0 ) || ( m_value.find( pattern, 0, caseSensitive ) > 0 );
}

#include "bibtexstring.moc"
