/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef _KBIBTEXPART_H_
#define _KBIBTEXPART_H_

#include <kparts/part.h>
#include <kparts/factory.h>

#include <selectaction.h>
#include <bibtexfile.h>

class QWidget;
class QComboBox;
class QFile;
class QPainter;
class QListViewItem;
class QProgressDialog;
class KURL;
class KListView;
class KRecentFilesAction;
class KAction;
class KBibTeXListView;
class KBibTeXSettingsDlg;
class BibTeXElement;
class KBibTeXActionTextSearch;

class KBibTeXPart : public KParts::ReadWritePart
{
    Q_OBJECT

public:
    KBibTeXPart( QWidget *parentWidget, const char *widgetName,
                 QObject *parent, const char *name );

    virtual ~KBibTeXPart();

    virtual void setReadWrite( bool rw );

    virtual void setModified( bool modified );

    void readSettings();
    void writeSettings();

    void insertText( QString &text );

public slots:
    void slotPreferences();

protected:
    virtual bool openFile();
    virtual bool saveFile();

private slots:
    void fileOpen();
    void fileInsert();
    bool fileSaveAs();
    bool fileSaveSelectedAs();
    void slotNewElement();
    bool editElement( QListViewItem* item = NULL );
    bool editElement( BibTeXElement* element );
    void deleteElement();
    void showProgress( int current, int total );
    void cutElement();
    void copyElement();
    void pasteElement();
    void showBibtexListContextMenu( KListView *, QListViewItem *, const QPoint &p );
    void slotSelectionChanged();
    void slotSearchGoogle();
    void slotSearchGoogleScholar();
    void slotSearchCiteseer();

private:
    enum InternetSearchEngine { iseGoogle, iseGoogleScholar, iseCiteSeer };
    enum SearchDirection { sdForward, sdBackward };
    enum SaveStatus { ssSave, ssExport, ssFailed };

    KBibTeXListView *m_bibtexlist;
    BibTeXFile *m_bibtexfile;
    int m_newElementCount;
    QProgressDialog *m_progressDialog;
    KRecentFilesAction *m_recent;
    KAction *m_actionFileInsert;
    KAction *m_actionFileSaveSelectedAs;
    KAction *m_actionElement;
    KAction *m_actionEditElement;
    KAction *m_actionDeleteElement;
    KAction *m_actionEditCut;
    KAction *m_actionEditCopy;
    KAction *m_actionEditPaste;
    KAction *m_actionEditSearchInternetGoogle;
    KAction *m_actionEditSearchInternetGoogleScholar;
    KAction *m_actionEditSearchInternetCiteseer;
    KAction *m_actionPreferences;
    KBibTeXActionTextSearch *m_actionTextSearch;
    KBibTeX::SelectAction *m_actionElementsList;
    KBibTeXSettingsDlg *m_settingsDlg;

    bool openFile( QFile &file, const KURL &visibleURL, bool insertOnly = FALSE );
    bool saveSelectedAs( const KURL& url );
    SaveStatus saveFile ( const QString& fileName, BibTeXFile *bibtexfile, BibTeXFile::Format bibtexFileFormat );
    bool saveFile ( const KURL& url, BibTeXFile *bibtexfile , BibTeXFile::Format bibtexFileFormat );
    void setupGUI( QWidget *parentWidget, const char *name );
    void setupActions();
    void updateMenu();
    void searchInternet( BibTeXElement *element, InternetSearchEngine engine );
    bool queryClose();

signals:
    void progressCancel();
};

class KInstance;
class KAboutData;

class KBibTeXPartFactory : public KParts::Factory
{
    Q_OBJECT
public:
    KBibTeXPartFactory();
    virtual ~KBibTeXPartFactory();
    virtual KParts::Part* createPartObject( QWidget *parentWidget, const char *widgetName,
                                            QObject *parent, const char *name,
                                            const char *classname, const QStringList & /* args */ );
    static KInstance* instance();

private:
    static KInstance* s_instance;
    static KAboutData* s_about;
};

#endif // _KBIBTEXPART_H_
