/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef KBIBTEXENTRYWIDGET_H
#define KBIBTEXENTRYWIDGET_H

#include <qwidget.h>
#include <qdialog.h>

class QComboBox;
class QFrame;
class QLineEdit;
class QListBox;
class QListBoxItem;
class QListView;
class QListViewItem;
class QPushButton;
class QSpinBox;
class QStringList;
class QTabWidget;
class QTextEdit;
class BibTeXEntry;
class BibTeXFile;
class KBibTeXEntryWidgetTab;
class KBibTeXEntryWidgetSource;

class KBibTeXEntryWidget : public QWidget
{
    Q_OBJECT

public:
    KBibTeXEntryWidget( BibTeXEntry *entry, BibTeXFile *bibtexfile, bool isReadWrite, QWidget *parent = 0, const char *name = 0 );

    ~KBibTeXEntryWidget();

    static QDialog::DialogCode execute( BibTeXEntry *entry, BibTeXFile *bibtexfile, const QString& caption, bool isReadWrite, QWidget *parent = 0, const char *name = 0 );

    friend class KBibTeXEntryWidgetTab;
    friend class KBibTeXEntryWidgetTitle;
    friend class KBibTeXEntryWidgetAuthor;

protected:
    BibTeXEntry::EntryType currentEntryType();
    bool eventFilter( QObject* /* object */, QEvent *event );

private:
    enum WidgetStatus {wsRequired, wsOptional, wsUnused};

    QTabWidget* m_tabWidgetMain;
    QComboBox* m_comboBoxEntryType;
    QLineEdit* m_lineEditID;
    KBibTeXEntryWidgetTab *m_tabTitle, *m_tabAuthor, *m_tabPublication, *m_tabMisc, *m_tabUserFields;
    KBibTeXEntryWidgetSource *m_tabSource;
    QListView *m_listViewWarnings;

    BibTeXEntry *m_entry;
    BibTeXFile *m_bibtexfile;
    bool m_enableAllFields;
    bool m_isReadWrite;

    void setupGUI();
    void setupEntryTypes();
    void getEntryData( BibTeXEntry *entry , bool fromSource = false );
    bool setEntryDataSpinBox( BibTeXEntry *entry, QSpinBox *sbx, BibTeXEntryField::FieldType fieldType );
    bool setEntryDataText( BibTeXEntry *entry, QString text, BibTeXEntryField::FieldType fieldType );
    bool setEntryData( BibTeXEntry *entry );
    void changeEntryType( BibTeXEntry::EntryType entryType );
    void setWidgetStatus( QWidget* widget, BibTeXEntryField::FieldRequireStatus requiredStatus );
    void refreshSourceView();
    void parseSourceView();

private slots:
    void slotOk();
    void slotEntryTypeChanged( int index );
    void slotEnableAllFields( bool enable );
    void slotCurrentTabChanged( QWidget* newTab );
    void warningsExecute( QListViewItem* );

public slots:
    void updateWarnings();
};

class KBibTeXEntryWidgetWarningsItem : public QListViewItem
{
public:
    enum WarningLevel {wlInformation = 1, wlWarning = 2, wlError = 3};

    KBibTeXEntryWidgetWarningsItem( WarningLevel level, const QString &message, QWidget *widget, QListView *parent, const char* name = NULL );
    ~KBibTeXEntryWidgetWarningsItem();

    void updateItem();
    QWidget *widget();

private:
    WarningLevel m_level;
    QString m_message;
    QWidget *m_widget;

};

#endif
