/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qevent.h>
#include <qdragobject.h>
#include <qapplication.h>
#include <qclipboard.h>
#include <qheader.h>
#include <qtextstream.h>

#include <klocale.h>
#include <kurl.h>
#include <kurldrag.h>

#include <kbibtex_part.h>
#include <kbibtexlistviewitem.h>
#include <bibtexfile.h>
#include <bibtexelement.h>
#include <bibtexentry.h>
#include <bibtexstring.h>
#include <bibtexcomment.h>
#include <bibtexpreamble.h>
#include <settings.h>
#include "kbibtexlistview.h"

KBibTeXListView::KBibTeXListView( KBibTeXPart *kbibtexPart, QWidget *parent, const char *name )
        : KListView( parent, name ), m_kbibtexPart( kbibtexPart ), m_bibtexFile( NULL )
{
    for ( int i = 0; i < ColumnsCount; i++ )
    {
        addColumn( Columns[ i ] );
        setColumnWidthMode( i, QListView::Manual );
    }

    setAllColumnsShowFocus( true );
    setShowSortIndicator( true );
    setSelectionMode( QListView::Extended );
    header() ->setClickEnabled ( TRUE );

    setAcceptDrops( TRUE );
    //     clipper() ->setAcceptDrops( TRUE );
    setDragEnabled( TRUE );
}

KBibTeXListView::~KBibTeXListView()
{
    // TODO
}

void KBibTeXListView::setBibTexFile( BibTeXFile *bibtexFile )
{
    m_bibtexFile = bibtexFile;
}

void KBibTeXListView::updateView( KBibTeXListViewItem *item )
{
    if ( m_bibtexFile != NULL )
    {
        QApplication::setOverrideCursor( Qt::waitCursor );
        setUpdatesEnabled( FALSE );

        if ( item == NULL )
        {
            clearAll();
            for ( unsigned int i = 0; i < m_bibtexFile->count(); i++ )
            {
                BibTeXElement *element = m_bibtexFile->at( i );
                if ( m_filter.isEmpty() || element->containsPattern( m_filter ) )
                    new KBibTeXListViewItem( element, this );
            }
            sort();
        }
        else
            item->updateItem();

        setUpdatesEnabled( TRUE );
        QApplication::restoreOverrideCursor();
        triggerUpdate();
    }
}

BibTeXFile *KBibTeXListView::selectedElements()
{
    BibTeXFile * result = new BibTeXFile();
    QListViewItemIterator it( this, QListViewItemIterator::Selected );
    while ( it.current() )
    {
        KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( it.current() );
        it++;

        BibTeXElement* originalElement = kblvi->element();

        BibTeXEntry* originalEntry = dynamic_cast<BibTeXEntry*>( originalElement );
        if ( originalEntry != NULL )
            result->appendElement( new BibTeXEntry( originalEntry ) );
        else
        {
            BibTeXComment* originalComment = dynamic_cast<BibTeXComment*>( originalElement );
            if ( originalComment != NULL )
                result->appendElement( new BibTeXComment( originalComment ) );
            else
            {
                BibTeXString* originalString = dynamic_cast<BibTeXString*>( originalElement );
                if ( originalString != NULL )
                    result->appendElement ( new BibTeXString( originalString ) );
                else
                {
                    BibTeXPreamble* originalPreamble = dynamic_cast<BibTeXPreamble*>( originalElement );
                    if ( originalPreamble != NULL )
                        result->appendElement( new BibTeXPreamble( originalPreamble ) );
                }
            }
        }
    }

    return result;
}

void KBibTeXListView::enabledActions()
{
    connect( header(), SIGNAL( sizeChange( int, int, int ) ), this, SLOT( columnSizeChange() ) );
    connect( header(), SIGNAL( clicked( int ) ), this, SLOT( setSortingColumn( int ) ) );
}

void KBibTeXListView::filterText( const QString& text )
{
    m_filter = text;
    updateView();
}

void KBibTeXListView::setSortingColumn( int column )
{
    setSortColumn( column );
}

void KBibTeXListView::columnSizeChange()
{
    for ( int i = 0; i < 5; i++ )
        kbibtexsettings.mainListColumnWidth[ i ] = columnWidth( i );
}

void KBibTeXListView::dragEnterEvent( QDragEnterEvent * event )
{
    bool canAccept = QTextDrag::canDecode( event ) || QUriDrag::canDecode( event );
    qDebug( "dragEnterEvent: %s", canAccept ? "acc" : "rej" );
    event->accept( canAccept );
}

void KBibTeXListView::contentsDragEnterEvent( QDragEnterEvent * event )
{
    dragEnterEvent( event );
}

void KBibTeXListView::dropEvent( QDropEvent * event )
{
    QString text;
    QStrList urls;

    if ( QUriDrag::decode( event, urls ) && urls.count() > 0 )
    {
        KURL url( urls.first() );
        if ( url.isValid() )
            m_kbibtexPart->openURL( url );
    }
    else if ( QTextDrag::decode( event, text ) )
        m_kbibtexPart->insertText( text );
}

void KBibTeXListView::contentsDropEvent( QDropEvent * event )
{
    dropEvent( event );
}

void KBibTeXListView::startDrag()
{
    QString buffer;
    QListViewItemIterator it( this, QListViewItemIterator::Selected );
    while ( it.current() )
    {
        KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( it.current() );

        buffer.append( kblvi->element() ->toString() );
        buffer.append( "\n" );

        it++;
    }

    QDragObject *d = new QTextDrag( buffer, this );
    d->dragCopy();
    // do NOT delete d.
}

void KBibTeXListView::clearAll()
{
    QListViewItemIterator it( this );
    while ( it.current() )
    {
        delete it.current();
        it++;
    }
    clear();
}

bool KBibTeXListView::acceptDrag( QDropEvent *event ) const
{
    return QTextDrag::canDecode( event ) || QUriDrag::canDecode( event );
}

#include "kbibtexlistview.moc"
