/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlistview.h>
#include <klocale.h>

#include "kbibtexlistviewitem.h"
#include "bibtexelement.h"
#include "bibtexentry.h"
#include "bibtexcomment.h"
#include "bibtexpreamble.h"
#include "bibtexstring.h"

KBibTeXListViewItem::KBibTeXListViewItem( BibTeXElement *element, QListView *parent ) : KListViewItem( parent, parent->lastItem() ), m_element( element )
{
    setTexts();
}

KBibTeXListViewItem::KBibTeXListViewItem( BibTeXElement *element, QListView *parent, QListViewItem *after ) : KListViewItem( parent, after ), m_element( element )
{
    setTexts();
}

KBibTeXListViewItem::~KBibTeXListViewItem()
{}

BibTeXElement* KBibTeXListViewItem::element()
{
    return m_element;
}

void KBibTeXListViewItem::updateItem()
{
    setTexts();
}

void KBibTeXListViewItem::setTexts()
{
    BibTeXEntry * entry;
    entry = dynamic_cast<BibTeXEntry*>( m_element );
    if ( entry )
    {
        setText( 0, BibTeXEntry::entryTypeToString( entry->entryType() ) );
        setText( 1, entry->id() );

        BibTeXEntryField * field = NULL;
        field = entry->getField( BibTeXEntryField::ftTitle );
        if ( field != NULL )
            setText( 2, field->value().replace( '{', "" ).replace( '}', "" ) );
        else
        {
            field = entry->getField( BibTeXEntryField::ftBookTitle );
            if ( field != NULL )
                setText( 2, field->value().replace( '{', "" ).replace( '}', "" ) );
        }

        field = entry->getField( BibTeXEntryField::ftAuthor );
        if ( ( field != NULL ) && !field->value().isEmpty() )
            setText( 3, field->value().replace( '{', "" ).replace( '}', "" ) );
        else
        {
            field = entry->getField( BibTeXEntryField::ftEditor );
            if ( ( field != NULL ) && !field->value().isEmpty() )
                setText( 3, field->value().replace( '{', "" ).replace( '}', "" ) );
            else
            {
                field = entry->getField( BibTeXEntryField::ftPublisher );
                if ( ( field != NULL ) && !field->value().isEmpty() )
                    setText( 3, field->value().replace( '{', "" ).replace( '}', "" ) );
            }
        }

        field = entry->getField( BibTeXEntryField::ftYear );
        if ( field != NULL )
            setText( 4, field->value() );
    }
    else
    {
        BibTeXComment *comment;
        comment = dynamic_cast<BibTeXComment*>( m_element );
        if ( comment )
        {
            setText( 0, i18n( "Comment" ) );
            setText( 2, comment->comment() );
        }
        else
        {
            BibTeXPreamble *preamble;
            preamble = dynamic_cast<BibTeXPreamble*>( m_element );
            if ( preamble )
            {
                setText( 0, i18n( "Preamble" ) );
                setText( 2, preamble->preamble() );
            }
            else
            {
                BibTeXString *string;
                string = dynamic_cast<BibTeXString*>( m_element );
                if ( string )
                {
                    setText( 0, i18n( "String" ) );
                    setText( 1, string->key() );
                    setText( 2, string->value() );
                }
            }
        }
    }
}
